@@ -45,7 +45,8 @@ from . import _get_dist_file_data, get_cached_distribution, get_index, get_distr
 from .distribution_cache import DistributionCache
 
 
-def generate_distribution_caches(index, dist_names=None, preclean=False, ignore_local=False, debug=False):
+def generate_distribution_caches(index, dist_names=None, preclean=False,
+    ignore_local=False, include_source=False, debug=False):
     if os.path.isfile(index):
         index = 'file://' + os.path.abspath(index)
     index = get_index(index)
@@ -57,7 +58,8 @@ def generate_distribution_caches(index, dist_names=None, preclean=False, ignore_
     caches = {}
     for dist_name in dist_names:
         try:
-            cache = generate_distribution_cache(index, dist_name, preclean=preclean, ignore_local=ignore_local, debug=debug)
+            cache = generate_distribution_cache(index, dist_name, preclean=preclean,
+                ignore_local=ignore_local, include_source=include_source, debug=debug)
         except RuntimeError as e:
             errors.append(str(e))
             continue
@@ -67,10 +69,12 @@ def generate_distribution_caches(index, dist_names=None, preclean=False, ignore_
     return caches
 
 
-def generate_distribution_cache(index, dist_name, preclean=False, ignore_local=False, debug=False):
-    dist, cache = _get_cached_distribution(index, dist_name, preclean=preclean, ignore_local=ignore_local)
-    # fetch all manifests
-    print('- fetch missing manifests')
+def generate_distribution_cache(index, dist_name, preclean=False, ignore_local=False,
+                                include_source=False, debug=False):
+    dist, cache = _get_cached_distribution(index, dist_name, preclean=preclean, ignore_local=ignore_local,
+                                           include_source=include_source)
+
+    print('- fetch missing release manifests')
     errors = []
     for pkg_name in sorted(dist.release_packages.keys()):
         repo = dist.repositories[dist.release_packages[pkg_name].repository_name].release_repository
@@ -107,13 +111,61 @@ def generate_distribution_cache(index, dist_name, preclean=False, ignore_local=F
     if not debug:
         print('')
 
+    if include_source:
+        print('- fetch source repository manifests')
+        for repo_name in sorted(dist.repositories.keys()):
+            if dist.repositories[repo_name].source_repository:
+                source_repo_cache = dist.get_source_repo_package_xmls(repo_name)
+                if debug:
+                    print('  - fetch "%s"' % repo_name)
+                else:
+                    sys.stdout.write('.')
+                    sys.stdout.flush()
+            else:
+                if debug:
+                    print('  - skip "%s" since it has no source entry.' % repo_name)
+                continue
+
+    if not debug:
+        print('')
+
     if errors:
         raise RuntimeError('\n'.join(errors))
 
     return cache
 
 
-def _get_cached_distribution(index, dist_name, preclean=False, ignore_local=False):
+class CacheYamlDumper(yaml.SafeDumper):
+    """ A yaml dumper specific to dumping the serialized rosdistro cache file.
+
+    Allows long lines and direct unicode representation. This avoids writing escape
+    sequences, line continuations, and other noise into the cache file. Also permits
+    long strings to alias each other (by default only objects do).
+    """
+
+    def __init__(self, *args, **kwargs):
+        kwargs['width'] = 10000
+        kwargs['allow_unicode'] = True
+        super(CacheYamlDumper, self).__init__(*args, **kwargs)
+
+    def ignore_aliases(self, content):
+        """ Allow strings that look like package XML to alias to each other in the YAML output. """
+        try:
+            basestring
+        except NameError:
+            # Python 3
+            basestring = str
+        return not (isinstance(content, basestring) and '<package' in content)
+
+    def represent_mapping(self, tag, mapping, flow_style=False):
+        """ Gives compact representation for the distribution_file section, while allowing the package
+            XML cache sections room to breathe."""
+        if any([x in mapping for x in ('source', 'release', 'doc')]):
+            flow_style = True
+        return yaml.SafeDumper.represent_mapping(self, tag, mapping, flow_style)
+
+
+def _get_cached_distribution(index, dist_name, preclean=False, ignore_local=False, include_source=False):
     print('Build cache for "%s"' % dist_name)
     cache = None
     try:
@@ -146,7 +198,11 @@ def _get_cached_distribution(index, dist_name, preclean=False, ignore_local=Fals
         # since format 2 of the index file might contain a single value rather then a list
         if not isinstance(rel_file_data, list):
             rel_file_data = [rel_file_data]
-        # update cache with current distribution file
+        # if we're not including the source portion of the cache, strip it out of the existing cache
+        # in order to skip the potentially lengthy cache invalidation process.
+        if not include_source:
+            cache.source_repo_package_xmls = {}
+        # update cache with current distribution file, which filters existing cache by validity.
         cache.update_distribution(rel_file_data)
     else:
         print('- build cache from scratch')
